@@ -325,6 +325,9 @@ static void
 giop_thread_free (GIOPThread *tdata)
 {
 	GList *l;
+
+	if (tdata == giop_main_thread)
+		giop_main_thread = NULL;
 	
 	if (giop_thread_safe ()) {
 		g_mutex_lock (giop_pool_hash_lock);
@@ -335,17 +338,23 @@ giop_thread_free (GIOPThread *tdata)
 	}
 	
 	g_list_free (tdata->keys);
+	tdata->keys = NULL;
 	
 	g_mutex_free (tdata->lock);
+	tdata->lock = NULL;
 	g_cond_free (tdata->incoming);
+	tdata->incoming = NULL;
+
 #ifdef G_ENABLE_DEBUG
 	if (tdata->async_ents)
 		g_warning ("Leaked async ents");
 	if (tdata->request_queue)
 		g_warning ("Leaked request queue");
 #endif
-	if (tdata->invoke_policies)
+	if (tdata->invoke_policies) {
 		g_queue_free (tdata->invoke_policies);
+		tdata->invoke_policies = NULL;
+	}
 	
 	g_free (tdata);
 }
